@@ -53,7 +53,7 @@ from functools import partial
 from GPIO import *
 from GPIO import cleanup as _cleanup_orig
 
-VERSION = "0.6.4"
+VERSION = "0.7.1"
 
 # BCM numbering mode by default
 setmode(BCM)
@@ -74,20 +74,19 @@ _is_waiting_for_interrupts = False
 # Internals
 _epoll = select.epoll()
 _SYS_GPIO_ROOT = '/sys/class/gpio/'
-_RPI_SYSINFO = None
 GPIO_FUNCTIONS = {0: "OUTPUT", 1: "INPUT", 4: "ALT0", 7: "-"}
 MODEL_DATA = {
-    '0002': ('B', '1.0', 256, '?'),
-    '0003': ('B', '1.0', 256, '?'),
-    '0004': ('B', '2.0', 256, 'Sony'),
-    '0005': ('B', '2.0', 256, 'Qisda'),
-    '0006': ('B', '2.0', 256, 'Egoman'),
-    '0007': ('A', '2.0', 256, 'Egoman'),
-    '0008': ('A', '2.0', 256, 'Sony'),
-    '0009': ('A', '2.0', 256, 'Qisda'),
-    '000d': ('B', '2.0', 512, 'Egoman'),
-    '000e': ('B', '2.0', 512, 'Sony'),
-    '000f': ('B', '2.0', 512, 'Qisda')
+    '2': ('B', '1.0', 256, '?'),
+    '3': ('B', '1.0', 256, '?'),
+    '4': ('B', '2.0', 256, 'Sony'),
+    '5': ('B', '2.0', 256, 'Qisda'),
+    '6': ('B', '2.0', 256, 'Egoman'),
+    '7': ('A', '2.0', 256, 'Egoman'),
+    '8': ('A', '2.0', 256, 'Sony'),
+    '9': ('A', '2.0', 256, 'Qisda'),
+    'd': ('B', '2.0', 512, 'Egoman'),
+    'e': ('B', '2.0', 512, 'Sony'),
+    'f': ('B', '2.0', 512, 'Qisda')
 }
 
 # Pin layout with GPIO numbers (pin-id range is 1 - 26)
@@ -100,30 +99,17 @@ PIN_TO_GPIO_LAYOUT_REV2 = (_DC3V3, _DC5V, 2, _DC5V, 3, _GND, 4, 14, _GND, 15, \
         17, 18, 27, _GND, 22, 23, _DC3V3, 24, 10, _GND, 9, 25, 11, 8, _GND, 7)
 
 
-def get_rpi_sysinfo():
+def rpi_sysinfo():
     """ Returns (model, revision, mb-ram and maker) for this raspberry """
-    global _RPI_SYSINFO
-    if _RPI_SYSINFO:
-        return _RPI_SYSINFO
-
-    import re
-    with open("/proc/cpuinfo") as f:
-        cpuinfo = f.read()
-    rev_hex = re.search(r"(?<=\nRevision)[ |:|\t]*\w+", cpuinfo) \
-            .group().strip(" :\t")
-    _RPI_SYSINFO = MODEL_DATA[rev_hex]
-    return _RPI_SYSINFO
+    return MODEL_DATA[RPI_REVISION_HEX.lstrip("0")]
 
 
-def is_valid_gpio_id(gpio_id, board_rev=None):
+def is_valid_gpio_id(gpio_id):
     """
-    Returns True/False depending on whether the supplied gpio_id is valid on
-    the current board. board_rev is either '1.0' or '2.0'
+    Returns True if the supplied gpio_id is valid on this board, else False
     """
-    if not board_rev:
-        board_rev = get_rpi_sysinfo()[1]
     return int(gpio_id) in (PIN_TO_GPIO_LAYOUT_REV1 \
-            if board_rev == "1.0" else PIN_TO_GPIO_LAYOUT_REV2)
+            if RPI_REVISION == 1 else PIN_TO_GPIO_LAYOUT_REV2)
 
 
 def _threaded_callback(callback, *args):
@@ -163,6 +149,7 @@ def add_interrupt_callback(gpio_id, callback, edge='both',
     if not os.path.exists(path_gpio):
         with open(_SYS_GPIO_ROOT + "export", "w") as f:
             f.write("%s" % gpio_id)
+        global _gpio_kernel_interfaces_created
         _gpio_kernel_interfaces_created.append(gpio_id)
         debug("- kernel interface created for GPIO %s" % gpio_id)
 
@@ -244,15 +231,20 @@ def wait_for_interrupts(epoll_timeout=1):
     """
     global _is_waiting_for_interrupts
     _is_waiting_for_interrupts = True
-    while _is_waiting_for_interrupts:
-        events = _epoll.poll(epoll_timeout)
-        for fileno, event in events:
-            if event & select.EPOLLPRI:
-                f = _map_fileno_to_file[fileno]
-                # read() is workaround for not getting new values with read(1)
-                val = f.read().strip()
-                f.seek(0)
-                _handle_interrupt(fileno, val)
+    try:
+        while _is_waiting_for_interrupts:
+            events = _epoll.poll(epoll_timeout)
+            for fileno, event in events:
+                if event & select.EPOLLPRI:
+                    f = _map_fileno_to_file[fileno]
+                    # read() is workaround for not getting new values
+                    # with read(1)
+                    val = f.read().strip()
+                    f.seek(0)
+                    _handle_interrupt(fileno, val)
+    except:
+        _cleanup_interfaces()
+        raise
 
 
 def stop_waiting_for_interrupts():
@@ -269,9 +261,11 @@ def _cleanup_interfaces():
     Remove all /sys/class/gpio/gpioN interfaces that this script created.
     Does not usually need to be used.
     """
+    debug("Cleaning up interfaces...")
     global _gpio_kernel_interfaces_created
     for gpio_id in _gpio_kernel_interfaces_created:
         # Remove the kernel GPIO interface
+        debug("- unexporting GPIO %s" % gpio_id)
         with open(_SYS_GPIO_ROOT + "unexport", "w") as f:
             f.write("%s" % gpio_id)
 
